@@ -1,6 +1,10 @@
-# Copyright (c) 2011-13, Mitchell Cooper
+#
+# Copyright (c) 2011-14, Mitchell Cooper
+#
 # Evented::Object: a simple yet featureful base class event framework.
-package Evented::Object::EventFire;
+# https://github.com/cooper/evented-object
+#
+package Evented::Object::EventFire; # leave this package name the same FOREVER.
  
 use warnings;
 use strict;
@@ -12,57 +16,64 @@ use 5.010;
 ##########################
 
 our $VERSION = $Evented::Object::VERSION;
-
 our $events  = $Evented::Object::events;
 our $props   = $Evented::Object::props;
 
 # create a new event object.
 sub new {
     my ($class, %opts) = @_;
+    $opts{callback_i} ||= 0;
     return bless { $props => \%opts }, $class;
 }
 
 # cancel all future callbacks once.
+# if stopped already, returns the reason. TODO: document that.
 sub stop {
-    shift->{$props}{stop} = 1;
+    my ($fire, $reason) = @_;
+    $fire->{$props}{stop} ||= $reason || 'unspecified';
 }
 
 # returns a true value if the given callback has been called.
 # with no argument, returns number of callbacks called so far.
 sub called {
-    my ($event, $callback) = @_;
+    my ($fire, $callback) = @_;
     
     # return the number of callbacks called.
     # this includes the current callback.
     if (!defined $callback) {
-        return scalar(keys %{$event->{$props}{called}}) + 1;
+        my $called = scalar keys %{ $fire->{$props}{called} };
+        $called++ unless $fire->{$props}{complete};
+        return $called;
     }
     
     # return whether the specified callback was called.
-    return $event->{$props}{called}{$callback};
+    return $fire->{$props}{called}{$callback};
     
 }
 
 # returns a true value if the given callback will be called soon.
 # with no argument, returns number of callbacks pending.
 sub pending {
-    my ($event, $callback) = @_;
+    my ($fire, $cb_name) = @_;
     
     # return number of callbacks remaining.
-    if (!defined $callback) {
-        return scalar $event->_pending_callbacks;
+    if (!defined $cb_name) {
+        return scalar $fire->_pending_callbacks;
     }
 
     # return whether the specified callback is pending.
-    return scalar grep { $_->callback_name eq $callback } $event->_pending_callbacks;
+    foreach my $callback ($fire->_pending_callbacks) {
+        return 1 if $callback->[2]{name} eq $cb_name;
+    }
     
+    return;
 }
 
 # cancels a future callback once.
 sub cancel {
-    my ($event, $callback) = @_;
-    $event->{$props}{cancelled}{$callback} = 1;
-    $event->{$props}{cancellor}{$callback} = $event->callback_name;
+    my ($fire, $callback) = @_;
+    $fire->{$props}{cancelled}{$callback} = 1;
+    $fire->{$props}{cancellor}{$callback} = $fire->callback_name;
     return 1;
 }
 
@@ -71,8 +82,8 @@ sub cancel {
 # if the return value has a possibility of being undef,
 # the only way to be sure is to first test ->callback_called.
 sub return_of {
-    my ($event, $callback) = @_;
-    return $event->{$props}{return}{$callback};
+    my ($fire, $callback) = @_;
+    return $fire->{$props}{return}{$callback};
 }
 
 # returns the callback that was last called.
@@ -102,7 +113,7 @@ sub callback_name {
 
 # returns the caller(1) value of ->fire_event().
 sub caller {
-    @{shift->{$props}{caller}};
+    @{ shift->{$props}{caller} };
 }
 
 # returns the priority of the callback being called.
@@ -123,30 +134,22 @@ sub object {
 # internal use only.
 # returns an array of the callbacks to come.
 sub _pending_callbacks {
-    my ($event, @pending) = shift;
+    my ($fire, @pending) = shift;
+    my $ef_props   = $fire->{$props};
+    my @collection = @{ $ef_props->{collection} };
     
-    # fetch iteration values.
-    my ($priority_index, $callback_index) = ($event->{$props}{priority_i}, $event->{$props}{callback_i});
-    my @callbacks  = @{$event->{$props}{current_callback_set}};
-    my @priorities = @{$event->{$props}{current_priority_set}};
+    # this is the last callback.
+    return @pending if $ef_props->{callback_i} >= $#collection;
     
-    # if $callback_index != $#callbacks, there are more callbacks in this priority.
-    if ($callback_index < $#callbacks) {
-        push @pending, @callbacks[$callback_index + 1 .. $#callbacks];
-    }
-    
-    # this is the last priority.
-    return @pending if $priority_index >= $#priorities;
-    
-    # for each remaining priority, insert all callbacks.
-    foreach my $priority (@priorities[$priority_index + 1 .. $#priorities]) {
-        push @pending, @{$event->object->{$events}{$event->event_name}{$priority}};
-    }
+    # get the remaining callbacks.
+    my $next_callback = $ef_props->{callback_i} + 1;
+    my @callbacks     = @collection[$next_callback..$#collection];
     
     # filter out any cancelled callbacks.
     my @filtered;
-    foreach my $cb (@callbacks) {
-        push @filtered, $cb unless $event->{$props}{cancelled}{$cb->{name}};
+    foreach my $callback (@callbacks) {
+        my $name = $callback->[2]{name};
+        push @filtered, $callback unless $ef_props->{cancelled}{$name};
     }
     
     # return the pending callbacks.
@@ -154,4 +157,14 @@ sub _pending_callbacks {
     
 }
 
-1
+###############
+### ALIASES ###
+###############
+
+sub object;
+
+BEGIN {
+    *eo = *object;
+}
+
+1;
